@@ -3,6 +3,7 @@ import shutil
 import textwrap
 from datetime import datetime
 from functools import partial
+from io import StringIO
 from unittest.mock import patch
 
 import pytest
@@ -21,6 +22,15 @@ class TestPkgdevCommitParseArgs:
         out, err = capsys.readouterr()
         assert err.strip() == 'pkgdev commit: error: not in ebuild repo'
 
+    def test_bad_repo_cwd(self, make_repo, capsys, tool):
+        repo = make_repo(masters=('nonexistent',))
+        with pytest.raises(SystemExit) as excinfo, \
+                chdir(repo.location):
+            tool.parse_args(['commit'])
+        assert excinfo.value.code == 2
+        out, err = capsys.readouterr()
+        assert err.strip().startswith('pkgdev commit: error: repo init failed')
+
     def test_non_git_repo_cwd(self, repo, capsys, tool):
         with pytest.raises(SystemExit) as excinfo, \
                 chdir(repo.location):
@@ -39,6 +49,23 @@ class TestPkgdevCommitParseArgs:
         out, err = capsys.readouterr()
         assert err.strip() == 'pkgdev commit: error: not in ebuild git repo'
 
+    def test_commit_signing(self, repo, make_git_repo, tool):
+        git_repo = make_git_repo(repo.location)
+        repo.create_ebuild('cat/pkg-0')
+        git_repo.add_all('cat/pkg-0', commit=False)
+        # signed commits aren't enabled by default
+        with chdir(repo.location):
+            options, _ = tool.parse_args(['commit', '-u'])
+            assert '--signoff' not in options.commit_args
+            assert '--gpg-sign' not in options.commit_args
+        # signed commits enabled by layout.conf setting
+        with open(pjoin(git_repo.path, 'metadata/layout.conf'), 'a+') as f:
+            f.write('sign-commits = true\n')
+        with chdir(repo.location):
+            options, _ = tool.parse_args(['commit', '-u'])
+            assert '--signoff' in options.commit_args
+            assert '--gpg-sign' in options.commit_args
+
     def test_git_commit_args(self, repo, make_git_repo, tool):
         git_repo = make_git_repo(repo.location)
         repo.create_ebuild('cat/pkg-0')
@@ -58,10 +85,53 @@ class TestPkgdevCommitParseArgs:
         git_repo = make_git_repo(repo.location)
         repo.create_ebuild('cat/pkg-0')
         git_repo.add_all('cat/pkg-0', commit=False)
-        author_opt = '--author="A U Thor <author@example.com>"'
         with chdir(repo.location):
-            options, _ = tool.parse_args(['commit', author_opt])
-        assert options.commit_args == [author_opt]
+            for opt in ('--author="A U Thor <author@example.com>"', '-e'):
+                options, _ = tool.parse_args(['commit', opt])
+            assert options.commit_args == [opt]
+
+    def test_scan_args(self, repo, make_git_repo, tool):
+        git_repo = make_git_repo(repo.location)
+        repo.create_ebuild('cat/pkg-0')
+        git_repo.add_all('cat/pkg-0', commit=False)
+        # pkgcheck isn't run in verbose mode by default
+        with chdir(repo.location):
+            options, _ = tool.parse_args(['commit'])
+        assert '-v' not in options.scan_args
+        # verbosity level is passed down to pkgcheck
+        with chdir(repo.location):
+            options, _ = tool.parse_args(['commit', '-v'])
+        assert '-v' in options.scan_args
+
+    def test_commit_tags(self, capsys, repo, make_git_repo, tool):
+        git_repo = make_git_repo(repo.location)
+        repo.create_ebuild('cat/pkg-0')
+        git_repo.add_all('cat/pkg-0', commit=False)
+        with chdir(repo.location):
+            # bug IDs
+            for opt in ('-b', '--bug'):
+                options, _ = tool.parse_args(['commit', opt, '1'])
+                assert options.footer == {('Bug', 'https://bugs.gentoo.org/1')}
+            # bug URLs
+            for opt in ('-b', '--bug'):
+                options, _ = tool.parse_args(['commit', opt, 'https://bugs.gentoo.org/2'])
+                assert options.footer == {('Bug', 'https://bugs.gentoo.org/2')}
+            # bug IDs
+            for opt in ('-c', '--closes'):
+                options, _ = tool.parse_args(['commit', opt, '1'])
+                assert options.footer == {('Closes', 'https://bugs.gentoo.org/1')}
+            # bug URLs
+            for opt in ('-c', '--closes'):
+                options, _ = tool.parse_args(['commit', opt, 'https://bugs.gentoo.org/2'])
+                assert options.footer == {('Closes', 'https://bugs.gentoo.org/2')}
+            # bad URL
+            for opt in ('-b', '-c'):
+                with pytest.raises(SystemExit) as excinfo:
+                    tool.parse_args(['commit', opt, 'bugs.gentoo.org/1'])
+                assert excinfo.value.code == 2
+                out, err = capsys.readouterr()
+                assert not out
+                assert 'invalid URL: bugs.gentoo.org/1' in err
 
 
 class TestPkgdevCommit:
@@ -86,6 +156,94 @@ class TestPkgdevCommit:
         assert not out
         assert err.strip() == 'pkgdev commit: error: no staged changes exist'
 
+    def test_git_message_opts(self, repo, make_git_repo, tmp_path):
+        """Verify message-related options are passed through to `git commit`."""
+        git_repo = make_git_repo(repo.location)
+        repo.create_ebuild('cat/pkg-0')
+        git_repo.add_all('cat/pkg-0', commit=False)
+        path = str(tmp_path / 'msg')
+        with open(path, 'w') as f:
+            f.write('commit1')
+
+        with patch('sys.argv', self.args + ['-u', '-F', path]), \
+                pytest.raises(SystemExit) as excinfo, \
+                chdir(git_repo.path):
+            self.script()
+        assert excinfo.value.code == 0
+        commit_msg = git_repo.log(['-1', '--pretty=tformat:%B', 'HEAD'])
+        assert commit_msg == ['commit1']
+
+        repo.create_ebuild('cat/pkg-1')
+        git_repo.add_all('cat/pkg-1', commit=False)
+        with os_environ(GIT_EDITOR="sed -i '1s/1/2/'"), \
+                patch('sys.argv', self.args + ['-u', '-t', path]), \
+                pytest.raises(SystemExit) as excinfo, \
+                chdir(git_repo.path):
+            self.script()
+        assert excinfo.value.code == 0
+        commit_msg = git_repo.log(['-1', '--pretty=tformat:%B', 'HEAD'])
+        assert commit_msg == ['commit2']
+
+    def test_message_template(self, capsys, repo, make_git_repo, tmp_path):
+        git_repo = make_git_repo(repo.location)
+        repo.create_ebuild('cat/pkg-0')
+        git_repo.add_all('cat/pkg-0')
+        path = str(tmp_path / 'msg')
+
+        # auto-generate prefix
+        with open(path, 'w') as f:
+            f.write(textwrap.dedent("""\
+                *: summary
+
+                body
+            """))
+
+        for i, opt in enumerate(['-M', '--message-template'], 1):
+            repo.create_ebuild(f'cat/pkg-{i}')
+            git_repo.add_all(f'cat/pkg-{i}', commit=False)
+            with patch('sys.argv', self.args + ['-u', opt, path]), \
+                    pytest.raises(SystemExit) as excinfo, \
+                    chdir(git_repo.path):
+                self.script()
+            assert excinfo.value.code == 0
+            commit_msg = git_repo.log(['-1', '--pretty=tformat:%B', 'HEAD'])
+            assert commit_msg == ['cat/pkg: summary', '', 'body']
+
+        # override prefix
+        with open(path, 'w') as f:
+            f.write(textwrap.dedent("""\
+                prefix: summary
+
+                body
+            """))
+
+        for i, opt in enumerate(['-M', '--message-template'], 3):
+            repo.create_ebuild(f'cat/pkg-{i}')
+            git_repo.add_all(f'cat/pkg-{i}', commit=False)
+            with patch('sys.argv', self.args + ['-u', opt, path]), \
+                    pytest.raises(SystemExit) as excinfo, \
+                    chdir(git_repo.path):
+                self.script()
+            assert excinfo.value.code == 0
+            commit_msg = git_repo.log(['-1', '--pretty=tformat:%B', 'HEAD'])
+            assert commit_msg == ['prefix: summary', '', 'body']
+
+        # empty message
+        with open(path, 'w') as f:
+            f.write('')
+
+        for i, opt in enumerate(['-M', '--message-template'], 5):
+            repo.create_ebuild(f'cat/pkg-{i}')
+            git_repo.add_all(f'cat/pkg-{i}', commit=False)
+            with patch('sys.argv', self.args + ['-u', opt, path]), \
+                    pytest.raises(SystemExit) as excinfo, \
+                    chdir(git_repo.path):
+                self.script()
+            assert excinfo.value.code == 2
+            out, err = capsys.readouterr()
+            assert not out
+            assert err.strip().startswith('pkgdev commit: error: empty message template')
+
     def test_custom_unprefixed_message(self, capsys, repo, make_git_repo):
         git_repo = make_git_repo(repo.location)
         ebuild_path = repo.create_ebuild('cat/pkg-0')
@@ -205,6 +363,17 @@ class TestPkgdevCommit:
                 f.write('# comment\n')
         assert commit().startswith('profiles/arch: ')
 
+        # single repo root file change
+        with open(pjoin(repo.location, 'skel.ebuild'), 'a+') as f:
+            f.write('# comment\n')
+        assert commit().startswith('skel.ebuild: ')
+
+        # multiple repo root file change (no commit message prefix)
+        for file in ('skel.ebuild', 'header.txt'):
+            with open(pjoin(repo.location, file), 'a+') as f:
+                f.write('# comment\n')
+        assert commit() == 'msg'
+
         # treewide changes (no commit message prefix)
         repo.create_ebuild('foo/bar-1')
         with open(pjoin(repo.location, 'eclass', 'foo.eclass'), 'a+') as f:
@@ -233,19 +402,31 @@ class TestPkgdevCommit:
         repo.create_ebuild('cat/newpkg-0')
         assert commit() == 'cat/newpkg: initial import'
 
-        # single bump
+        # single addition
         repo.create_ebuild('cat/pkg-1')
-        assert commit() == 'cat/pkg: bump 1'
+        assert commit() == 'cat/pkg: add 1'
 
-        # multiple bumps
+        # multiple additions
         repo.create_ebuild('cat/pkg-2')
         repo.create_ebuild('cat/pkg-3')
-        assert commit() == 'cat/pkg: bump 2, 3'
+        assert commit() == 'cat/pkg: add 2, 3'
 
-        # large number of bumps in a single commit
+        # large number of additions in a single commit
         for v in range(10000, 10010):
             repo.create_ebuild(f'cat/pkg-{v}')
-        assert commit() == 'cat/pkg: bump versions'
+        assert commit() == 'cat/pkg: add versions'
+
+        # create Manifest
+        with open(pjoin(git_repo.path, 'cat/pkg/Manifest'), 'w') as f:
+            f.write('DIST pkg-3.tar.gz 101 BLAKE2B deadbeef SHA512 deadbeef\n')
+        assert commit() == 'cat/pkg: update Manifest'
+        # update Manifest
+        with open(pjoin(git_repo.path, 'cat/pkg/Manifest'), 'a+') as f:
+            f.write('DIST pkg-2.tar.gz 101 BLAKE2B deadbeef SHA512 deadbeef\n')
+        assert commit() == 'cat/pkg: update Manifest'
+        # remove Manifest
+        os.remove(pjoin(git_repo.path, 'cat/pkg/Manifest'))
+        assert commit() == 'cat/pkg: update Manifest'
 
         # single removal
         os.remove(pjoin(git_repo.path, 'cat/pkg/pkg-3.ebuild'))
@@ -292,11 +473,20 @@ class TestPkgdevCommit:
         # but they can be forcibly mangled
         with open(ebuild_path, 'a+') as f:
             f.write('# comment')
-        commit(['-M', '-u', '-m', 'mangling'])
+        commit(['--mangle', '-u', '-m', 'mangling'])
         # mangled pre-commit, file now ends with newline
         with open(ebuild_path) as f:
             assert f.read()[-1] == '\n'
 
+        # FILESDIR content is ignored even when forced
+        path = pjoin(os.path.dirname(ebuild_path), 'files', 'pkg.patch')
+        os.makedirs(os.path.dirname(path))
+        with open(path, 'w') as f:
+            f.write('# comment')
+        # verify file doesn't end with newline
+        with open(path) as f:
+            assert f.read()[-1] != '\n'
+
     def test_gentoo_file_mangling(self, make_repo, make_git_repo):
         repo = make_repo(repo_id='gentoo')
         git_repo = make_git_repo(repo.location)
@@ -322,6 +512,15 @@ class TestPkgdevCommit:
         with open(ebuild_path) as f:
             assert f.read()[-1] == '\n'
 
+        # FILESDIR content is ignored
+        path = pjoin(os.path.dirname(ebuild_path), 'files', 'pkg.patch')
+        os.makedirs(os.path.dirname(path))
+        with open(path, 'w') as f:
+            f.write('# comment')
+        # verify file doesn't end with newline
+        with open(path) as f:
+            assert f.read()[-1] != '\n'
+
         for years, org in (
                 ('1999-2020', 'Gentoo Authors'),
                 ('1999-2020', 'Gentoo Foundation'),
@@ -359,7 +558,7 @@ class TestPkgdevCommit:
             out, err = capsys.readouterr()
             assert err == out == ''
             commit_msg = git_repo.log(['-1', '--pretty=tformat:%B', 'HEAD'])
-            assert commit_msg == [f'cat/pkg: bump {i}']
+            assert commit_msg == [f'cat/pkg: add {i}']
 
     def test_failed_scan(self, capsys, repo, make_git_repo):
         git_repo = make_git_repo(repo.location)
@@ -386,7 +585,8 @@ class TestPkgdevCommit:
         """)
 
         # ignore failures to create the commit
-        with patch('sys.argv', self.args + ['--scan', '--ignore-failures']), \
+        with patch('sys.argv', self.args + ['--scan', '--ask']), \
+                patch('sys.stdin', StringIO('y\n')), \
                 pytest.raises(SystemExit) as excinfo, \
                 chdir(git_repo.path):
             self.script()
